/*
Monetra v7.x C API

Copyright (c) 2008, Main Street Softworks, Inc.
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, 
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of the Main Street Softworks, Inc. nor the names of its
      contributors may be used to endorse or promote products derived from 
      this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __MONETRA_H__
#define __MONETRA_H__

#ifndef __MYCONFIG_H__
#  define __MYCONFIG_H__
#  ifdef _WIN32
#    include "libmonetra_w32config.h"
#  else
#    include "libmonetra_config.h"
#  endif
#endif

#define LIBMONETRA_VERSION 070003


#ifdef _WIN32
#  ifdef LIBMONETRA_INTERNAL
#    define M_EXPORT __declspec(dllexport)
#  else /* !LIBMONETRA_INTERNAL */
#    define M_EXPORT __declspec(dllimport)
#  endif /* !LIBMONETRA_INTERNAL */
#  ifdef LIBMONETRA_SPEC_STDCALL
#    define M_SPEC __stdcall
#  else
#    define M_SPEC __cdecl
#  endif
#else /* !WIN32 */
#  define M_EXPORT
#  define M_SPEC
#endif /* !WIN32 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "libmonetra_compat.h"

#define M_Safe(a) (a==NULL?"":a)
#define M_isdigit(a) ((a >= '0' && a <= '9')?1:0)

/* Define libmonetra types */
#ifdef HAVE_STDINT_H
#  include <stdint.h>
   typedef int64_t M_int64;
   typedef uint64_t M_uint64;
   typedef int32_t M_int32;
   typedef uint32_t M_uint32;
   typedef uintptr_t M_uintptr;
#else /* NO STDINT.H */
#  ifdef _WIN32
     typedef unsigned __int64 M_uint64;
     typedef signed __int64 M_int64;
     typedef signed int M_int32;
     typedef unsigned int M_uint32;
#    ifdef _WIN64
       typedef M_uint64 M_uintptr;
#    else /* _WIN64 */
       typedef M_uint32 M_uintptr;
#    endif /* _WIN64 */
#  else /* !WIN32 */
#    ifndef HAVE_LONGLONG
       typedef unsigned long M_uint64;
       typedef signed long M_int64;
#    else /* ! HAVE_LONGLONG */
       typedef unsigned long long M_uint64;
       typedef signed long long M_int64;
#    endif /* HAVE_LONGLONG */
     typedef signed int M_int32;
     typedef unsigned int M_uint32;
     typedef unsigned long M_uintptr;
#  endif /* _WIN32 */
#endif /* HAVE_STDINT_H */

M_int64 M_atoint64(const char *string);

#if defined(HAVE_ATOLL) && ! defined(__APPLE__) 
/* Mac OS X 10.2 does not support atoll() but 10.3 does. We want 10.2 compat */
#define M_atoll(a)	atoll(a)
#elif defined(HAVE_STRTOLL)
#define M_atoll(a)	strtoll(a, NULL, 10)
#else 
/* Windows should fall through to here */
#define M_atoll(a)	M_atoint64(a)
#endif

/* Transaction States */
#define M_UNUSED 0
#define M_NEW 100
#define M_PENDING 1
#define M_DONE 2

/* Transaction Results */
#ifdef M_ERROR
#undef M_ERROR
#endif
#define M_ERROR -1
#define M_FAIL 0
#define M_SUCCESS 1
#define M_AUTH 2
#define M_DENY 3
#define M_CALL 4
#define M_DUPL 5
#define M_PKUP 6
#define M_RETRY 7
#define M_SETUP 8
#define M_TIMEOUT 9

// AVS/CVV2 response codes
#define M_GOOD 1
#define M_BAD 0
#define M_STREET 2
#define M_ZIP 3
#define M_UNKNOWN -1

/* SSL retries */
#ifndef M_SSL_RETRIES
#define	M_SSL_RETRIES	5
#endif

typedef void * M_CONN;

/* BASE ROUTINES */
enum m_ssllock_style {
	M_SSLLOCK_NONE = 0,	/* No locking method defined, should not be set by a user */
	M_SSLLOCK_EXTERNAL = 1,	/* OpenSSL's locks are guaranteed to be
				 * initialized externally, OpenSSL itself
				 * would also be initialized extrenally */
	M_SSLLOCK_INTERNAL = 2	/* LibMonetra is responsible for all OpenSSL
				 * code.  OpenSSL should not be used outside
				 * of libmonetra calls in an application if this
				 * is used, especially in multi-threaded code */
};
M_EXPORT int M_SPEC M_InitEngine_ex(enum m_ssllock_style lockstyle);
M_EXPORT int M_SPEC M_InitEngine(const char *location);
M_EXPORT void M_SPEC M_DestroyEngine(void);
M_EXPORT void M_SPEC M_InitConn(M_CONN *myconn);

M_EXPORT int M_SPEC M_SetProxyHost(M_CONN *myconn, const char *host, unsigned short port, const char *type);
M_EXPORT int M_SPEC M_SetProxyTimeout(M_CONN *myconn, int timeout);
M_EXPORT int M_SPEC M_SetProxyUser(M_CONN *myconn, const char *username, const char *password);
M_EXPORT int M_SPEC M_SetProxyLocalNets(M_CONN *myconn, const char *localnets, char *error, int errlen);

M_EXPORT int M_SPEC M_SetBlocking(M_CONN *myconn, int tf);
M_EXPORT int M_SPEC M_SetLogging(M_CONN *myconn, int level);
M_EXPORT int M_SPEC M_SetTimeout(M_CONN *myconn, long timeout);
#ifdef M_DROPFILE_ENABLED
M_EXPORT int M_SPEC M_SetDropFile(M_CONN *myconn, const char *df_location);
#endif
M_EXPORT int M_SPEC M_SetIP(M_CONN *myconn, const char *host, unsigned short port);
M_EXPORT int M_SPEC M_SetSSL(M_CONN *myconn, const char *host, unsigned short port);
M_EXPORT int M_SPEC M_SetSSL_CAfile(M_CONN *myconn, const char *path);
M_EXPORT int M_SPEC M_SetSSL_Files(M_CONN *myconn, const char *sslkeyfile, const char *sslcertfile);
M_EXPORT void M_SPEC M_VerifyConnection(M_CONN *myconn, int tf);
M_EXPORT void M_SPEC M_VerifySSLCert(M_CONN *myconn, int tf);
M_EXPORT int M_SPEC M_ValidateIdentifier(M_CONN * myconn, int tf);
M_EXPORT int M_SPEC M_Connect(M_CONN *myconn);
M_EXPORT void M_SPEC M_MaxConnTimeout(M_CONN *myconn, int maxtime);
M_EXPORT const char * M_SPEC M_ConnectionError(M_CONN *myconn);
M_EXPORT void M_SPEC M_DestroyConn(M_CONN *myconn);
M_EXPORT int M_SPEC M_Monitor(M_CONN *myconn);
M_EXPORT int M_SPEC M_Monitor_ex(M_CONN *myconn, long wait_us);
M_EXPORT int M_SPEC M_TransactionsSent(M_CONN *myconn);

#define M_DeleteResponse(a, b) M_DeleteTrans(a, b)
M_EXPORT void M_SPEC M_DeleteTrans(M_CONN *myconn, M_uintptr identifier);

M_EXPORT M_uintptr M_SPEC M_TransNew(M_CONN *myconn);
M_EXPORT int M_SPEC M_TransKeyVal(M_CONN * myconn, M_uintptr identifier, const char *key, 
			    const char *value);
M_EXPORT int M_SPEC M_TransBinaryKeyVal(M_CONN *myconn, M_uintptr identifier, const char *key, const char *value, int value_len);

#ifndef LIBMONETRA_SPEC_STDCALL
M_EXPORT int M_SPEC M_TransParam(M_CONN *myconn, M_uintptr identifier, int key, ...);
#endif

M_EXPORT int M_SPEC M_TransSend(M_CONN *myconn, M_uintptr identifier);
M_EXPORT const char * M_SPEC M_ResponseParam(M_CONN *myconn, M_uintptr identifier, const char *key);
M_EXPORT char ** M_SPEC M_ResponseKeys(M_CONN *myconn, M_uintptr identifier, int *num_keys);
M_EXPORT const char * M_SPEC M_ResponseKeys_index(char **keys, int num_keys, int idx);
M_EXPORT int M_SPEC M_FreeResponseKeys(char **keys, int num_keys);


/* Response Checking Routines */
M_EXPORT int M_SPEC M_ReturnStatus(M_CONN *myconn, M_uintptr identifier);
M_EXPORT int M_SPEC M_ReturnCode(M_CONN *myconn, M_uintptr identifier);
M_EXPORT long M_SPEC M_TransactionItem(M_CONN *myconn, M_uintptr identifier);
M_EXPORT long M_SPEC M_TransactionBatch(M_CONN *myconn, M_uintptr identifier);
M_EXPORT M_int64 M_SPEC M_TransactionID(M_CONN *myconn, M_uintptr identifier);
M_EXPORT const char * M_SPEC M_TransactionAuth(M_CONN *myconn, M_uintptr identifier);
M_EXPORT const char * M_SPEC M_TransactionText(M_CONN *myconn, M_uintptr identifier);
M_EXPORT int M_SPEC M_TransactionAVS(M_CONN *myconn, M_uintptr identifier);
M_EXPORT int M_SPEC M_TransactionCV(M_CONN *myconn, M_uintptr identifier);
M_EXPORT long M_SPEC M_TransInQueue(M_CONN *myconn);
M_EXPORT int M_SPEC M_CheckStatus(M_CONN *myconn, M_uintptr identifier);
M_EXPORT long M_SPEC M_CompleteAuthorizations(M_CONN *myconn, M_uintptr **listings);
M_EXPORT M_uintptr M_SPEC M_CompleteAuthorizations_index(M_uintptr *listings, int num_listings, int idx);
M_EXPORT int M_SPEC M_FreeCompleteAuthorizations(M_uintptr *listings, int num_listings);

/* Comma Delimited Routines */
M_EXPORT int M_SPEC M_IsCommaDelimited(M_CONN *myconn, M_uintptr identifier);
M_EXPORT int M_SPEC M_ParseCommaDelimited(M_CONN *myconn, M_uintptr identifier);
M_EXPORT const char * M_SPEC M_GetCommaDelimited(M_CONN *myconn, M_uintptr identifier);
M_EXPORT const char * M_SPEC M_GetCell(M_CONN *myconn, M_uintptr identifier, const char *column, long row);
M_EXPORT const char * M_SPEC M_GetCellByNum(M_CONN *myconn, M_uintptr identifier, int column, long row);
M_EXPORT char * M_SPEC M_GetBinaryCell(M_CONN *myconn, M_uintptr identifier, const char *column, long row, int *out_length);
M_EXPORT void M_SPEC M_FreeBinaryCell(char *cell);
M_EXPORT int M_SPEC M_NumColumns(M_CONN *myconn, M_uintptr identifier);
M_EXPORT long M_SPEC M_NumRows(M_CONN *myconn, M_uintptr identifier);
M_EXPORT const char * M_SPEC M_GetHeader(M_CONN *myconn, M_uintptr identifier, int column_num);

M_EXPORT int M_SPEC M_uwait(unsigned long length);

/* TEXT RESPONSE CONVERSIONS */

M_EXPORT const char * M_SPEC M_TEXT_Code(int code);
M_EXPORT const char * M_SPEC M_TEXT_AVS(int code);
M_EXPORT const char * M_SPEC M_TEXT_CV(int code);

/* generate digest of SSL certificate for use in client-cert restrictions */
M_EXPORT char * M_SPEC M_SSLCert_gen_hash(const char *filename);

#ifdef _WIN32
/* Specifically make sure callbacks are __cdecl, not stdcall */
typedef void * (__cdecl *M_Register_Mutex)(void);
typedef int (__cdecl *M_Mutex_Lock)(void *);
typedef int (__cdecl *M_Mutex_Unlock)(void *);
typedef int (__cdecl *M_Unregister_Mutex)(void *);
typedef unsigned long (__cdecl *M_ThreadID)(void);
#else
typedef void * (M_SPEC *M_Register_Mutex)(void);
typedef int (M_SPEC *M_Mutex_Lock)(void *);
typedef int (M_SPEC *M_Mutex_Unlock)(void *);
typedef int (M_SPEC *M_Unregister_Mutex)(void *);
typedef unsigned long (M_SPEC *M_ThreadID)(void);
#endif

M_EXPORT int M_SPEC M_Register_mutexinit(M_CONN *myconn, M_Register_Mutex reg);
M_EXPORT int M_SPEC M_Register_mutexdestroy(M_CONN *myconn, M_Unregister_Mutex reg);
M_EXPORT int M_SPEC M_Register_mutexlock(M_CONN *myconn, M_Mutex_Lock reg);
M_EXPORT int M_SPEC M_Register_mutexunlock(M_CONN *myconn, M_Mutex_Unlock reg);
M_EXPORT int M_SPEC M_Register_threadid(M_CONN *myconn, M_ThreadID reg);
M_EXPORT int M_SPEC M_EnableThreadSafety(M_CONN *myconn);

#include "libmonetra_legacy.h"
#include "libmonetra_defines.h"


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif
